@@ -23,7 +23,7 @@ import six
 
 from ..describe import Description, describeRoute
 from ..rest import Resource, RestException, filtermodel, loadmodel
-from ...constants import AccessType
+from ...constants import AccessType, TokenScope
 from girder.models.model_base import AccessException, GirderException
 from girder.api import access
 
@@ -48,8 +48,9 @@ class File(Resource):
         self.route('POST', (':id', 'copy'), self.copy)
         self.route('PUT', (':id',), self.updateFile)
         self.route('PUT', (':id', 'contents'), self.updateFileContents)
+        self.route('PUT', (':id', 'move'), self.moveFileToAssetstore)
 
-    @access.public
+    @access.public(scope=TokenScope.DATA_READ)
     @loadmodel(model='file', level=AccessType.READ)
     @filtermodel(model='file')
     @describeRoute(
@@ -61,7 +62,7 @@ class File(Resource):
     def getFile(self, file, params):
         return file
 
-    @access.user
+    @access.user(scope=TokenScope.DATA_WRITE)
     @describeRoute(
         Description('Start a new upload or create an empty or link file.')
         .responseClass('Upload')
@@ -76,6 +77,8 @@ class File(Resource):
         .param('reference', 'If included, this information is passed to the '
                'data.process event when the upload is complete.',
                required=False)
+        .param('assetstoreId', 'Direct the upload to a specific assetstore.',
+               required=False)
         .errorResponse()
         .errorResponse('Write access was denied on the parent folder.', 403)
         .errorResponse('Failed to create upload.', 500)
@@ -107,11 +110,15 @@ class File(Resource):
                     parentType=parentType, creator=user), user)
         else:
             self.requireParams('size', params)
+            assetstore = None
+            if params.get('assetstoreId'):
+                assetstore = self.model('assetstore').load(
+                    params['assetstoreId'])
             try:
                 upload = self.model('upload').createUpload(
                     user=user, name=params['name'], parentType=parentType,
                     parent=parent, size=int(params['size']), mimeType=mimeType,
-                    reference=params.get('reference'))
+                    reference=params.get('reference'), assetstore=assetstore)
             except OSError as exc:
                 if exc.errno == errno.EACCES:
                     raise GirderException(
@@ -124,7 +131,7 @@ class File(Resource):
                 return self.model('file').filter(
                     self.model('upload').finalizeUpload(upload), user)
 
-    @access.user
+    @access.user(scope=TokenScope.DATA_WRITE)
     @describeRoute(
         Description('Finalize an upload explicitly if necessary.')
         .notes('This is only required in certain non-standard upload '
@@ -157,7 +164,7 @@ class File(Resource):
         extraKeys = file.get('additionalFinalizeKeys', ())
         return self.model('file').filter(file, user, additionalKeys=extraKeys)
 
-    @access.user
+    @access.user(scope=TokenScope.DATA_WRITE)
     @describeRoute(
         Description('Request required offset before resuming an upload.')
         .param('uploadId', 'The ID of the upload record.')
@@ -182,7 +189,7 @@ class File(Resource):
         else:
             return offset
 
-    @access.user
+    @access.user(scope=TokenScope.DATA_WRITE)
     @describeRoute(
         Description('Upload a chunk of a file with multipart/form-data.')
         .consumes('multipart/form-data')
@@ -233,7 +240,7 @@ class File(Resource):
             raise
 
     @access.cookie
-    @access.public
+    @access.public(scope=TokenScope.DATA_READ)
     @loadmodel(model='file', level=AccessType.READ)
     @describeRoute(
         Description('Download a file.')
@@ -251,6 +258,8 @@ class File(Resource):
         .param('contentDisposition', 'Specify the Content-Disposition response '
                'header disposition-type value', required=False,
                enum=['inline', 'attachment'], default='attachment')
+        .param('extraParameters', 'Arbitrary data to send along with the '
+               'download request.', required=False)
         .errorResponse('ID was invalid.')
         .errorResponse('Read access was denied on the parent folder.', 403)
     )
@@ -279,11 +288,14 @@ class File(Resource):
             raise RestException('Unallowed contentDisposition type "%s".' %
                                 contentDisp)
 
+        extraParameters = params.get('extraParameters')
+
         return self.model('file').download(file, offset, endByte=endByte,
-                                           contentDisposition=contentDisp)
+                                           contentDisposition=contentDisp,
+                                           extraParameters=extraParameters)
 
     @access.cookie
-    @access.public
+    @access.public(scope=TokenScope.DATA_READ)
     @describeRoute(
         Description('Download a file.')
         .param('id', 'The ID of the file.', paramType='path')
@@ -300,7 +312,7 @@ class File(Resource):
     def downloadWithName(self, id, name, params):
         return self.download(id=id, params=params)
 
-    @access.user
+    @access.user(scope=TokenScope.DATA_WRITE)
     @loadmodel(model='file', level=AccessType.WRITE)
     @describeRoute(
         Description('Delete a file by ID.')
@@ -311,7 +323,7 @@ class File(Resource):
     def deleteFile(self, file, params):
         self.model('file').remove(file)
 
-    @access.user
+    @access.user(scope=TokenScope.DATA_WRITE)
     @loadmodel(model='upload')
     @describeRoute(
         Description('Cancel a partially completed upload.')
@@ -328,7 +340,7 @@ class File(Resource):
         self.model('upload').cancelUpload(upload)
         return {'message': 'Upload canceled.'}
 
-    @access.user
+    @access.user(scope=TokenScope.DATA_WRITE)
     @loadmodel(model='file', level=AccessType.WRITE)
     @filtermodel(model='file')
     @describeRoute(
@@ -345,7 +357,7 @@ class File(Resource):
                                       (file.get('mimeType') or '').strip())
         return self.model('file').updateFile(file)
 
-    @access.user
+    @access.user(scope=TokenScope.DATA_WRITE)
     @loadmodel(model='file', level=AccessType.WRITE)
     @describeRoute(
         Description('Change the contents of an existing file.')
@@ -354,6 +366,8 @@ class File(Resource):
         .param('reference', 'If included, this information is passed to the '
                'data.process event when the upload is complete.',
                required=False)
+        .param('assetstoreId', 'Direct the upload to a specific assetstore.',
+               required=False)
         .notes('After calling this, send the chunks just like you would with a '
                'normal file upload.')
     )
@@ -361,10 +375,13 @@ class File(Resource):
         self.requireParams('size', params)
         user = self.getCurrentUser()
 
+        assetstore = None
+        if params.get('assetstoreId'):
+            assetstore = self.model('assetstore').load(params['assetstoreId'])
         # Create a new upload record into the existing file
         upload = self.model('upload').createUploadToFile(
             file=file, user=user, size=int(params['size']),
-            reference=params.get('reference'))
+            reference=params.get('reference'), assetstore=assetstore)
 
         if upload['size'] > 0:
             return upload
@@ -372,7 +389,21 @@ class File(Resource):
             return self.model('file').filter(
                 self.model('upload').finalizeUpload(upload), user)
 
-    @access.user
+    @access.user(scope=TokenScope.DATA_WRITE)
+    @loadmodel(model='file', level=AccessType.WRITE)
+    @describeRoute(
+        Description('Move a file to a different assetstore.')
+        .param('id', 'The ID of the file.', paramType='path')
+        .param('assetstoreId', 'The destination assetstore.')
+    )
+    def moveFileToAssetstore(self, file, params):
+        self.requireParams('assetstoreId', params)
+        user = self.getCurrentUser()
+        assetstore = self.model('assetstore').load(params['assetstoreId'])
+        return self.model('upload').moveFileToAssetstore(
+            file=file, user=user, assetstore=assetstore)
+
+    @access.user(scope=TokenScope.DATA_WRITE)
     @loadmodel(model='file', level=AccessType.READ)
     @loadmodel(model='item', map={'itemId': 'item'}, level=AccessType.WRITE)
     @filtermodel(model='file')
