@@ -22,11 +22,13 @@ import json
 
 from ..describe import Description, describeRoute
 from ..rest import Resource as BaseResource, RestException
-from girder.constants import AccessType
+from girder.constants import AccessType, TokenScope
 from girder.api import access
 from girder.models.model_base import AccessControlledModel
 from girder.utility import acl_mixin
+from girder.utility import parseTimestamp
 from girder.utility import ziputil
+from girder.utility import path as path_util
 from girder.utility.progress import ProgressContext
 
 # Plugins can modify this set to allow other types to be searched
@@ -43,6 +45,8 @@ class Resource(BaseResource):
         self.route('GET', ('search',), self.search)
         self.route('GET', ('lookup',), self.lookup)
         self.route('GET', (':id',), self.getResource)
+        self.route('GET', (':id', 'path'), self.path)
+        self.route('PUT', (':id', 'timestamp'), self.setTimestamp)
         self.route('GET', ('download',), self.download)
         self.route('POST', ('download',), self.download)
         self.route('PUT', ('move',), self.moveResources)
@@ -110,6 +114,7 @@ class Resource(BaseResource):
         Validate a JSON string listing resources.  The resources parameter is a
         JSON encoded dictionary with each key a model name and each value a
         list of ids that must be present in that model.
+
         :param params: a dictionary of parameters that must include 'resources'
         :param allowedModels: if present, an iterable of models that may be
                               included in the resources.
@@ -134,6 +139,7 @@ class Resource(BaseResource):
     def _getResourceModel(self, kind, funcName=None):
         """
         Load and return a model with a specific function or throw an exception.
+
         :param kind: the name of the model to load
         :param funcName: a function name to ensure that each model contains.
         :returns: the loaded model.
@@ -149,6 +155,7 @@ class Resource(BaseResource):
     def _lookUpToken(self, token, parentType, parent):
         """
         Find a particular child resource by name or throw an exception.
+
         :param token: the name of the child resource to find
         :param parentType: the type of the parent to search
         :param parent: the parent resource
@@ -180,8 +187,18 @@ class Resource(BaseResource):
         raise RestException('Child resource not found: %s(%s)->%s' % (
             parentType, parent.get('name', parent.get('_id')), token))
 
-    def _lookUpPath(self, path, user):
-        pathArray = [token for token in path.split('/') if token]
+    def _lookUpPath(self, path, user, test=False):
+        """
+        Look up a resource in the data hierarchy by path.
+
+        :param path: path of the resource
+        :param user: user with correct privileges to access path
+        :param test: defaults to false, when set to true
+            will return None instead of throwing exception when
+            path doesn't exist
+        """
+        path = path.lstrip('/')
+        pathArray = path_util.split(path)
         model = pathArray[0]
 
         parent = None
@@ -190,15 +207,21 @@ class Resource(BaseResource):
             parent = self.model('user').findOne({'login': username})
 
             if parent is None:
-                raise RestException('User not found: %s' % username)
+                if test:
+                    return None
+                else:
+                    raise RestException('User not found: %s' % username)
 
         elif model == 'collection':
             collectionName = pathArray[1]
             parent = self.model('collection').findOne({'name': collectionName})
 
             if parent is None:
-                raise RestException(
-                    'Collection not found: %s' % collectionName)
+                if test:
+                    return None
+                else:
+                    raise RestException(
+                        'Collection not found: %s' % collectionName)
 
         else:
             raise RestException('Invalid path format')
@@ -210,12 +233,15 @@ class Resource(BaseResource):
                 document, model = self._lookUpToken(token, model, document)
                 self.model(model).requireAccess(document, user)
         except RestException:
-            raise RestException('Path not found: %s' % path)
+            if test:
+                return None
+            else:
+                raise RestException('Path not found: %s' % path)
 
         result = self.model(model).filter(document, user)
         return result
 
-    @access.public
+    @access.public(scope=TokenScope.DATA_READ)
     @describeRoute(
         Description('Look up a resource in the data hierarchy by path.')
         .param('path',
@@ -223,16 +249,70 @@ class Resource(BaseResource):
                'path starting with either "/user/[user name]", for a user\'s '
                'resources or "/collection/[collection name]", for resources '
                'under a collection.')
+        .param('test',
+               'Specify whether to return None instead of throwing an '
+               'exception when path doesn\'t exist.',
+               required=False, dataType='boolean', default=False)
         .errorResponse('Path is invalid.')
         .errorResponse('Path refers to a resource that does not exist.')
         .errorResponse('Read access was denied for the resource.', 403)
     )
     def lookup(self, params):
         self.requireParams('path', params)
-        return self._lookUpPath(params['path'], self.getCurrentUser())
+        test = self.boolParam('test', params, default=False)
+        return self._lookUpPath(params['path'], self.getCurrentUser(), test)
+
+    def _getResourceName(self, type, doc):
+        if type == 'user':
+            return doc['login']
+        elif type in ('file', 'item', 'folder', 'user', 'collection'):
+            return doc['name']
+        else:
+            raise RestException(
+                'Invalid resource type.'
+            )
+
+    def _getResourceParent(self, type, doc):
+        if type == 'file':
+            return doc['itemId'], 'item'
+        elif type == 'item':
+            return doc['folderId'], 'folder'
+        elif type == 'folder':
+            return doc['parentId'], doc['parentCollection']
+        else:
+            raise RestException(
+                'Invalid resource type.'
+            )
+
+    @access.public(scope=TokenScope.DATA_READ)
+    @describeRoute(
+        Description('Get path of a resource.')
+        .param('id', 'The ID of the resource.', paramType='path')
+        .param('type', 'The type of the resource (item, file, etc.).')
+        .errorResponse('ID was invalid.')
+        .errorResponse('Invalid resource type.')
+        .errorResponse('Read access was denied for the resource.', 403)
+    )
+    def path(self, id, params):
+        path = []
+        type = params['type']
+        while True:
+            doc = self._getResource(id, type)
+            if doc is None:
+                raise RestException('Invalid resource id.')
+            name = self._getResourceName(type, doc)
+            path.insert(0, name)
+
+            if type in ('file', 'item', 'folder'):
+                id, type = self._getResourceParent(type, doc)
+            else:
+                break
+
+        path.insert(0, type)
+        return '/' + path_util.join(path)
 
     @access.cookie(force=True)
-    @access.public
+    @access.public(scope=TokenScope.DATA_READ)
     @describeRoute(
         Description('Download a set of items, folders, collections, and users '
                     'as a zip archive.')
@@ -286,7 +366,7 @@ class Resource(BaseResource):
             yield zip.footer()
         return stream
 
-    @access.user
+    @access.user(scope=TokenScope.DATA_OWN)
     @describeRoute(
         Description('Delete a set of items, folders, or other resources.')
         .param('resources', 'A JSON-encoded list of types to delete.  Each '
@@ -338,6 +418,14 @@ class Resource(BaseResource):
                             ctx.update(current=current,
                                        message='Deleted ' + kind)
 
+    def _getResource(self, id, type):
+        model = self._getResourceModel(type)
+        if (isinstance(model, (acl_mixin.AccessControlMixin,
+                               AccessControlledModel))):
+            user = self.getCurrentUser()
+            return model.load(id=id, user=user, level=AccessType.READ)
+        return model.load(id=id)
+
     @access.admin
     @describeRoute(
         Description('Get any resource by ID.')
@@ -347,12 +435,33 @@ class Resource(BaseResource):
         .errorResponse('Read access was denied for the resource.', 403)
     )
     def getResource(self, id, params):
+        return self._getResource(id, params['type'])
+
+    @access.admin
+    @describeRoute(
+        Description('Set the created or updated timestamp for a resource.')
+        .param('id', 'The ID of the resource.', paramType='path')
+        .param('type', 'The type of the resource (item, file, etc.).')
+        .param('created', 'The new created timestamp.', required=False)
+        .param('updated', 'The new updated timestamp.', required=False)
+        .errorResponse('ID was invalid.')
+        .errorResponse('Access was denied for the resource.', 403)
+    )
+    def setTimestamp(self, id, params):
+        user = self.getCurrentUser()
         model = self._getResourceModel(params['type'])
-        if (isinstance(model, (acl_mixin.AccessControlMixin,
-                               AccessControlledModel))):
-            user = self.getCurrentUser()
-            return model.load(id=id, user=user, level=AccessType.READ)
-        return model.load(id=id)
+        doc = model.load(id=id, user=user, level=AccessType.WRITE)
+        if not doc:
+            raise RestException('Resource not found.')
+        if 'created' in params:
+            if 'created' not in doc:
+                raise RestException('Resource has no "created" field.')
+            doc['created'] = parseTimestamp(params['created'])
+        if 'updated' in params:
+            if 'updated' not in doc:
+                raise RestException('Resource has no "updated" field.')
+            doc['updated'] = parseTimestamp(params['updated'])
+        return model.save(doc)
 
     def _prepareMoveOrCopy(self, params):
         user = self.getCurrentUser()
@@ -368,7 +477,7 @@ class Resource(BaseResource):
         progress = self.boolParam('progress', params, default=False)
         return user, resources, parent, parentType, progress
 
-    @access.user
+    @access.user(scope=TokenScope.DATA_WRITE)
     @describeRoute(
         Description('Move a set of items and folders.')
         .param('resources', 'A JSON-encoded list of types to move.  Each type '
@@ -417,7 +526,7 @@ class Resource(BaseResource):
                             model.move(doc, parent, parentType)
                     ctx.update(increment=1)
 
-    @access.user
+    @access.user(scope=TokenScope.DATA_WRITE)
     @describeRoute(
         Description('Copy a set of items and folders.')
         .param('resources', 'A JSON-encoded list of types to copy. Each type '
